
/**
* Manages the creation of Shrunken heads...I mean URLs
*/
var TinyUrl = {
	
	/**
	* Get the URL shrinking service the user has selected
	* @returns {TinyUrl.services.*}
	*/
	getService : function(){
		var service = TinyUrl.services.tinyurl;
		var prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("");
		
		// Check preferences
		if(prefs.prefHasUserValue("tinyurl.service")){
			var name = prefs.getCharPref("tinyurl.service");
			
			if(TinyUrl.services[name]){ // Valid?
				service = TinyUrl.services[name];
			}
		}
		
		return service;
	},
	
	
	/**
	* Shrinks the URL with the service which has been set in the preferences.
	* @param {TinyUrl.URL} url The long URL to shrink.
	* @param {Function} callback A function to call when the shrinking process is done.
	*							 + This function gets one parameters: A TinyUrl.URL details or FALSE if there was an error
	*/
	shrink : function(url, callback){
		var service = this.getService();
		url.service = service;
		service.shrink(url, callback);
	},
	
	/**
	* Asks the user for a URL alias.
	* @param {TinyUrl.URL} tinyUrl The long URL to shrink.
	* @param {boolean} repeat If the first alias failed and we're asking them again.
	*/
	promptAlias : function(tinyUrl, repeat){
		var msg = "Would you like to create an alias for the short URL?\n"+ tinyUrl.service.url +"/<alias>";
		
		// Different message if we're asking again
		if(repeat === true){
			msg = "That alias is taken, try another or click Cancel to get a generic short URL.\n"+ tinyUrl.service.url +"/<alias>";
		}
		
		return prompt(msg, tinyUrl.alias || "");
	},
	
	/**
	* Add the full URL tooltip to links that have a short url service in the HREF
	* @param {Anchor} anchor The anchor tag with a TinyURL
	* @param {Document} doc The page document object
	*/
	addPreviews : function(doc){
		
		// Create search pattern and services hash (services[url] = serviceObj)
		var name, service;
		var pattern = [];
		var services = {};
		for(name in TinyUrl.services){
			service = TinyUrl.services[name];
			services[service.url.toLowerCase()] = service;
			pattern[pattern.length] = "("+ service.url +")";
		}
		pattern = new RegExp("^"+ pattern.join("|"), "i");
		
		// Search through anchors
		var match, anchor, url;
		var anchors = doc.getElementsByTagName("a");
		for(var i = 0; i < anchors.length; i++){
			anchor = anchors[i];
			url = anchor.href;
			
			// Match pattern?
			if( (match = url.match(pattern)) != null){
			
				// Find URl in service hash
				service = services[match[0].toLowerCase()];
				if(service){
				
					// Expand and update the link
					service.expand(new TinyUrl.URL(null, url, anchor), function(tinyUrl){
						if(tinyUrl.anchor && tinyUrl.orig){
							tinyUrl.anchor.setAttribute("title", tinyUrl.orig);
						}
					});
				}
			}
		}
	},
	
	/**
	* For services which don't have an expand API, we can use the brute force
	* method of making a request for the URL to see where it redirects to
	* @param {TinyUrl.URL} url The short URL to expand
	* @param {Function} callback The function to call when the expanding process is done.
	*/
	expand : function(url, callback){
	
		// Setup request objects
		var io = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
		var uri = Components.classes["@mozilla.org/network/standard-url;1"].createInstance(Components.interfaces.nsIURI);
		uri.spec = url.tiny;
		var request = io.newChannelFromURI(uri).QueryInterface(Components.interfaces.nsIHttpChannel);
		
		request.redirectionLimit = 0;
		request.requestMethod = "HEAD";
		
		// Start request and find redirect location
		request.asyncOpen({
			onStartRequest : function(request, context){ },
			onStopRequest : function(request, context, statusCode){ 
				url.orig = request.getResponseHeader("Location");
				callback.call(request, url);
			},
			onDataAvailable : function(request, context, inputStream, offset, count){} 
		}, null);
		url.request = request;
	},
	
	/**
	* This is a wrapper for the standard shrinking API that many services offer
	* where the API URL simply returns the short URL as the complete response.
	* @params {String} apiUrl The API URL with all the parameters needed
	* @param {TinyUrl.URL} url The short URL to expand (this is purely for the callback function, the long URL needs to be part of the API url)
	* @param {Function} callback The function to call when the expanding process is done.
	*/
	stdServiceAPI : function(apiUrl, url, callback){
	
		// Setup Request
		var request	= new XMLHttpRequest();
		request.open("GET", apiUrl, true);
		request.setRequestHeader("Referer", "http://mozmonkey.com");
		
		// Handle request
		request.onload = {
			handleEvent : function(e){
				
				//Parse Response
				var tiny = request.responseText;
				if(tiny && tiny.length < 60 && tiny.toLowerCase().indexOf("error") < 0){
					url.tiny = tiny;
				}
				
				// Notify callback function
				callback.call(request, url);
			}
		};
		
		// Handle Error
		request.onerror = {
			handleEvent : function(e){
				callback.call(request, url);
			}
		}
		
		// Start Request
		request.send("");
		url.request = request;
	}
	
}

/**
* The shrunken URL details
*/
TinyUrl.URL = function(orig, tiny, anchor){

	/**
	* The original URL
	* @type {String}
	*/
	this.orig = orig;
	
	/**
	* The short URL
	* @type {String}
	*/
	this.tiny = tiny;
	
	/**
	* The URL alias the user has provided
	*/
	this.alias = null;
	
	/**
	* The anchor element with a short URL
	* @type {DomNode}
	*/
	this.anchor = anchor;
	
	/**
	* The title of the link or page
	* @type {String}
	*/
	this.title = "";
	
	/**
	* The XHR object processing this request
	*/
	this.request = window;
	
	/**
	* The shrinking service that was used
	*/
	this.service = null;
	
	/**
	* Abort the XHR request
	*/
	this.abort = function(){
		try{
			if(this.request != null){
				this.request.abort();
			}
		} catch(e){ }
	}
	
}

/**
* All the URL shrinking services this extension supports
*/
TinyUrl.services = {
	
	/**
	* Bit.ly
	*/
	bitly : {
		
		name : "Bit.ly",
		url : "http://bit.ly/",
		alias: false,
		
		/**
		* @see TinyUrl.shrink() for details.
		*/
		shrink : function(url, callback){
			url.service = this;
			var apiUrl = "http://bit.ly/api?url="+ encodeURIComponent(url.orig);
			
			// Extra callback for alias
			TinyUrl.stdServiceAPI(apiUrl, url, callback);
		},
		
		/**
		* @see TinyUrl.expand() for details
		*/
		expand : function(tiny, callback){
			tiny.service = this;
		
			var apiUrl = "http://bit.ly/resolve?url="+ encodeURIComponent(tiny.tiny);
			TinyUrl.stdServiceAPI(apiUrl, new TinyUrl.URL(), function(tinyUrl){
				tiny.orig = tinyUrl.tiny;
				callback.call(this, tiny);
			});
		}
	},
	
	/**
	* Is.gd
	*/
	isgd : {
		
		name : "Is.gd",
		url : "http://is.gd/",
		alias: false,
		
		/**
		* @see TinyUrl.shrink() for details.
		*/
		shrink : function(url, callback){
			url.service = this;
			var apiUrl = "http://is.gd/api.php?longurl="+ encodeURIComponent(url.orig);
			TinyUrl.stdServiceAPI(apiUrl, url, callback);
		},
		
		/**
		* @see TinyUrl.expand() for details
		*/
		expand : function(tiny, callback){
			tiny.service = this;
			TinyUrl.expand(tiny, callback);
		}
	},
	
	
	/**
	* Rurl.org
	*/
	rurl : {
		
		name : "Rurl.org",
		url : "http://rurl.org/",
		apikey : "b512878b00efff5439688dccebbd551c",
		alias: false,
		
		/**
		* @see TinyUrl.shrink() for details.
		*/
		shrink : function(url, callback){
			url.service = this;
			
			// Encode URL (Rurl.org will choke if you try to use encodeURIComponent)
			var urlencode = url.orig;
			urlencode = urlencode.replace(/\+/g, '%2B');
			urlencode = urlencode.replace(/\?/g, '%3F');
			urlencode = urlencode.replace(/&/g, '%26');
			urlencode = urlencode.replace(/=/g, '%3D');
			urlencode = urlencode.replace(/#/g, '%23');
			
			var request = new XMLHttpRequest();
			request.open("POST", "http://rurl.org/api/");
			request.setRequestHeader("Referer", "http://mozmonkey.com");
			request.setRequestHeader("Content-Type", "application/x-www-form-urlencoded");
			
			request.onload = {
				handleEvent : function(evt){
					// Get XML document
					var doc = request.responseXML;
					if(doc){
					
						// Get tiny URL
						var tiny = doc.getElementsByTagName("url");
						if(tiny.length > 0){
							url.tiny = tiny[0].firstChild.nodeValue;
						}
						
						callback.call(request, url);
					}
				}
			};
			
			// Handle Error
			request.onerror = {
				handleEvent : function(e){
					callback.call(request, url);
				}
			};
			
			request.send("url="+ urlencode +"&apikey="+ this.apikey +"&submit=Submit");
			url.request = request;
		},
		
		/**
		* @see TinyUrl.expand() for details
		*/
		expand : function(tiny, callback){
			tiny.service = this;
			TinyUrl.expand(tiny, callback);
		}
	},
	
		
	/**
	* snipr
	*/
	snipr : {
		
		name : "Snipr.com",
		url : "http://snipr.com",
		alias : true,
		
		/**
		* @see TinyUrl.shrink() for details.
		*/
		shrink : function(url, callback){
			var apiUrl = "http://snipr.com/site/snip?r=simple&link="+ encodeURIComponent(url.orig);
			TinyUrl.stdServiceAPI(apiUrl, url, callback);
		},
		
		/**
		* @see TinyUrl.expand() for details
		*/
		expand : function(tiny, callback){
			tiny.service = this;
			
			// Get ID
			var base = "http://snipr.com/".length;
			if(tiny.tiny.length <= base){
				return;
			}
			var id = tiny.tiny.substring(base);
			
			
			// Can standard api and swap URL results
			var apiUrl = "http://snipurl.com/resolveurl?id="+ id;
			TinyUrl.stdServiceAPI(apiUrl, new TinyUrl.URL(), function(tinyUrl){
				tiny.orig = tinyUrl.tiny;
				callback.call(this, tiny);
			});
		}
	},
	
		
	/**
	* Shrinkthislink
	*/
	shrunklink : {
		
		name : "Shrunklink.com",
		url : "http://shrunklink.com",
		alias: false,
		
		/**
		* @see TinyUrl.shrink() for details.
		*/
		shrink : function(url, callback){
			url.service = this;
			var apiUrl = "http://shrinkthislink.com/addplain.php?url="+ encodeURIComponent(url.orig);
			
			// Call service API and add http
			TinyUrl.stdServiceAPI(apiUrl, url, function(tinyUrl){
				if(tinyUrl.tiny){
					tinyUrl.tiny = "http://"+ tinyUrl.tiny;
					callback.call(this, tinyUrl);
				}
			});
		},
		
		/**
		* @see TinyUrl.expand() for details
		*/
		expand : function(tiny, callback){
			tiny.service = this;
			TinyUrl.expand(tiny, callback);
		}
	},

	/**
	* TinyUrl.com
	*/
	tinyurl : {
		
		name : "TinyUrl.com",
		url : "http://tinyurl.com/",
		alias: false,
		
		/**
		* @see TinyUrl.shrink() for details.
		*/
		shrink : function(url, callback){
			url.service = this;
			
			// Get short url
			var apiUrl = "http://tinyurl.com/api-create.php?url="+ encodeURIComponent(url.orig);;
			TinyUrl.stdServiceAPI(apiUrl, url, callback);
		},
		
		/**
		* @see TinyUrl.expand() for details
		*/
		expand : function(tiny, callback){
			tiny.service = this;
			TinyUrl.expand(tiny, callback);
		}
		
	},

	/**
	* Tiny.pl
	*/
	tinypl : {
		
		name : "Tiny.pl",
		url : "http://tiny.pl",
		alias: false,
		
		/**
		* @see TinyUrl.shrink() for details.
		*/
		shrink : function(url, callback){
			url.service = this;
			
			// Setup Request
			var request	= new XMLHttpRequest();
			request.open("POST", "http://tiny.pl", true);
			request.setRequestHeader("Referer", "http://mozmonkey.com");
			request.setRequestHeader("Content-Type", "application/x-www-form-urlencoded");
			
			// Handle request
			request.onload = {
				handleEvent : function(e){
					
					//Parse Response
					var html = request.responseText;
					var regexp = /<a id="tinyAdr" href="([^"]*)"/i;
					var match;
					if( (match = html.match(regexp)) != null){
						url.tiny = match[1];
					}
					
					callback.call(request, url);
				}
			};
			
			// Handle Error
			request.onerror = {
				handleEvent : function(e){
					callback.call(request, url);
				}
			}
			
			// Start Request
			request.send("pralina="+ encodeURIComponent(url.orig));
			url.request = request;
		},
		
		/**
		* @see TinyUrl.expand() for details
		*/
		expand : function(tiny, callback){
			tiny.service = this;
			TinyUrl.expand(tiny, callback);
		}
		
	},
	
		
	/**
	* Wapurl
	*/
	wapurl : {
		
		name : "Wapurl.co.uk",
		url : "http://wapurl.co.uk/",
		alias: false,
		
		/**
		* @see TinyUrl.shrink() for details.
		*/
		shrink : function(url, callback){
			url.service = this;
						
			var request = new XMLHttpRequest();
			request.open("GET", "http://wapURL.co.uk//api.cfm?URLToConvert="+ encodeURIComponent(url.orig) +"&urlEncoded=1");
			request.setRequestHeader("Referer", "http://mozmonkey.com");
			
			request.onload = {
				handleEvent : function(evt){
					// Get XML document
					var doc = request.responseXML;
					if(doc){
					
						// Get tiny URL
						var tiny = doc.getElementsByTagName("wapURL");
						if(tiny.length > 0 && tiny[1].firstChild.nodeValue.indexOf("http") > -1){
							url.tiny = tiny[1].firstChild.nodeValue;
						}
						
						callback.call(request, url);
					}
				}
			};
			
			// Handle Error
			request.onerror = {
				handleEvent : function(e){
					callback.call(request, url);
				}
			};
			
			request.send("");
			url.request = request;
		},
		
		/**
		* @see TinyUrl.expand() for details
		*/
		expand : function(tiny, callback){
			tiny.service = this;
			TinyUrl.expand(tiny, callback);
		}
	}
};
